package com.yunzhi.shigongli.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.BaseController;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.ResponseBean;
import com.yunzhi.shigongli.entity.TaRecommend;
import com.yunzhi.shigongli.service.ITaRecommendService;
import com.yunzhi.shigongli.vo.RecommendVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

/**
 * <p>
    * 系统推荐 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */

@Api(tags = "系统推荐")
@RestController
@RequestMapping("/")
public class TaRecommendController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaRecommendController.class);

    @Autowired
    public ITaRecommendService iTaRecommendService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/recommend",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = RecommendVO.class)
    public ResponseBean tdTypeList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                   @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{

        IPage<RecommendVO> pg = new Page<>(pageNum, pageSize);

        IPage<RecommendVO> result = iTaRecommendService.getPagedBy(pg);
        return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taRecommend 实体对象
     * @return
     */
    @RequestMapping(value="/admin/recommend",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean tdTypeAdd(@ApiParam("保存内容") @RequestBody TaRecommend taRecommend) throws Exception{

        taRecommend.setStatus(Constants.STATUS_NORMAL);
        taRecommend.setCreateDate(LocalDateTime.now());

        if (iTaRecommendService.save(taRecommend)){
            return ResponseBean.success(taRecommend);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/recommend/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean tdTypeDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        if(iTaRecommendService.removeById(id)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taRecommend 实体对象
     * @return
     */
    @RequestMapping(value="/admin/recommend/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean tdTypeUpdate(@ApiParam("对象ID") @PathVariable Integer id,
                                     @ApiParam("更新内容") @RequestBody TaRecommend taRecommend) throws Exception{

        TaRecommend origin = iTaRecommendService.getById(id);
        if (null == origin) {
            return ResponseBean.error("修改失败, 对象不存在", ResponseBean.ERROR_UNAVAILABLE);
        }

        // 只允许修改权重
        origin.setWeight(taRecommend.getWeight());

        if (iTaRecommendService.updateById(origin)){
            return ResponseBean.success(origin);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/recommend/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean tdTypeGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        return ResponseBean.success(iTaRecommendService.getById(id));
    }
}
