package com.yunzhi.shigongli.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.BaseController;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.ResponseBean;
import com.yunzhi.shigongli.common.StringUtils;
import com.yunzhi.shigongli.entity.TaResource;
import com.yunzhi.shigongli.service.ITaResourceService;
import com.yunzhi.shigongli.vo.ResourceListVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
    * 资源表 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */

@Api(tags = "资源表")
@RestController
@RequestMapping("/")
public class TaResourceController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaResourceController.class);

    @Autowired
    public ITaResourceService iTaResourceService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/wx/resource",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                @ApiParam("定位") @RequestParam(value ="location", required = false) String location,
                                @ApiParam("分类") @RequestParam(value ="typeId", required = false) String typeId,
                                @ApiParam("关键词") @RequestParam(value ="q", required = false) String q,
                                @ApiParam("资源类型") @RequestParam(value ="targetType", required = false) String targetType) throws Exception{

        IPage<ResourceListVO> pg = new Page<>(pageNum, pageSize);
        IPage<ResourceListVO> result = null;

        if (!StringUtils.isEmpty(q)) {
            // 如果使用的搜索, 则走搜索的方法
            result = iTaResourceService.searchList(pg, getCurrentPerson(), location, q, targetType);
        } else {
            result = iTaResourceService.getWxPagedBy(pg, getCurrentPerson(), location, typeId);
        }

        return ResponseBean.success(result);
    }

    @RequestMapping(value="/wx/resource/{targetType}/{targetId}",method= RequestMethod.GET)
    @ApiOperation(value="获取资源详情", notes = "获取资源详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getList(@ApiParam("资源类型") @PathVariable String targetType,
                                @ApiParam("资源对象ID") @PathVariable String targetId,
                                @ApiParam("定位") @RequestParam(value ="location", required = false) String location) throws Exception{

        ResourceListVO resourceListVO = iTaResourceService.getDetailByTarget(targetType, targetId, location);

        return ResponseBean.success(resourceListVO);
    }

    /**
     * 资源字典
     * @param pageNum
     * @param pageSize
     * @param targetName
     * @return
     * @throws Exception
     */
    @GetMapping("/admin/resource/dict")
    @ApiOperation(value="获取资源字典", notes = "获取资源字典", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getDict(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                @ApiParam("资源名称") @RequestParam(value = "targetName", required = false) String targetName) throws Exception {
        IPage<TaResource> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaResource> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", Constants.STATUS_NORMAL);
        queryWrapper.like(!StringUtils.isEmpty(targetName), "target_name", "%"+targetName+"%");
        queryWrapper.orderByDesc("weight");

        return ResponseBean.success(iTaResourceService.page(pg, queryWrapper));
    }


    /**
     * 资源推荐
     * @param pageNum
     * @param pageSize
     * @param location
     * @return
     * @throws Exception
     */
    @GetMapping("/wx/resource/recommend")
    @ApiOperation(value="资源推荐", notes = "资源推荐", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getRecommend(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                     @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                     @ApiParam("定位") @RequestParam(value = "location") String location) throws Exception {
        IPage<ResourceListVO> pg = new Page<>(pageNum, pageSize);

        IPage<ResourceListVO> result = iTaResourceService.getWxRecommend(pg, getCurrentPerson().getPersonId(), location);

        return ResponseBean.success(result);
    }
}
