package com.yunzhi.shigongli.controller;

import com.yunzhi.shigongli.common.BaseController;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.ResponseBean;
import com.yunzhi.shigongli.common.StringUtils;
import com.yunzhi.shigongli.entity.TaTravel;
import com.yunzhi.shigongli.entity.TaTravelItem;
import com.yunzhi.shigongli.entity.TaTravelPerson;
import com.yunzhi.shigongli.entity.TaUv;
import com.yunzhi.shigongli.service.ITaTravelItemService;
import com.yunzhi.shigongli.service.ITaTravelPersonService;
import com.yunzhi.shigongli.service.ITaTravelService;
import com.yunzhi.shigongli.service.ITaUvService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.models.auth.In;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

/**
 * <p>
    * 行程地点 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */

@Api(tags = "行程地点")
@RestController
@RequestMapping("/")
public class TaTravelItemController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaTravelItemController.class);

    @Autowired
    public ITaTravelService iTaTravelService;

    @Autowired
    public ITaTravelItemService iTaTravelItemService;

    @Autowired
    public ITaTravelPersonService iTaTravelPersonService;


    /**
     * 保存我的行程地点
     * @param id
     * @param taTravelItem
     * @return
     * @throws Exception
     */
    @PostMapping("/wx/travel/{id}/travel-item")
    @ApiOperation(value="保存我的行程地点", notes = "保存我的行程地点", httpMethod = "GET", response = TaTravel.class)
    public ResponseBean saveTravelItem(@ApiParam("行程ID") @PathVariable String id,
                                       @ApiParam("保存内容") @RequestBody TaTravelItem taTravelItem) throws Exception {
        TaTravelPerson taTravelPerson = iTaTravelPersonService.getByPerson(getCurrentPerson().getPersonId(), id);
        if (null == taTravelPerson) {
            return ResponseBean.error("未找到更新内容", ResponseBean.ERROR_UNAVAILABLE);
        }

        if (StringUtils.isEmpty(taTravelItem.getTargetType()) || StringUtils.isEmpty(taTravelItem.getTargetId())) {
            return ResponseBean.error("没有正确设置关联地点", ResponseBean.ERROR_UNAVAILABLE);
        }

        if (null == taTravelItem.getDayOrder() || taTravelItem.getDayOrder() < 1) {
            taTravelItem.setDayOrder(1);
        }

        // 获取最大序号
        Integer maxNo = iTaTravelItemService.getMaxNoOf(id, taTravelItem.getDayOrder());
        if (maxNo == null) {
            maxNo = 0;
        }

        taTravelItem.setTravelId(id);
        taTravelItem.setSortNo(maxNo + 1);

        if (iTaTravelItemService.save(taTravelItem)) {
            return ResponseBean.success(taTravelItem);
        } else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 删除我的行程地点
     * @param id
     * @param no
     * @return
     * @throws Exception
     */
    @DeleteMapping("/wx/travel/{id}/travel-item/{no}")
    @ApiOperation(value="删除我的行程地点", notes = "删除我的行程地点", httpMethod = "DELETE", response = TaTravel.class)
    public ResponseBean saveTravelItem(@ApiParam("行程ID") @PathVariable String id,
                                       @ApiParam("行程地点ID") @PathVariable String no) throws Exception {
        TaTravelPerson taTravelPerson = iTaTravelPersonService.getByPerson(getCurrentPerson().getPersonId(), id);
        if (null == taTravelPerson) {
            return ResponseBean.error("未找到更新内容", ResponseBean.ERROR_UNAVAILABLE);
        }

        if (iTaTravelItemService.removeById(no)) {
            return ResponseBean.success(null);
        } else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }
}
