package com.yunzhi.shigongli.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.DateUtils;
import com.yunzhi.shigongli.entity.TaHotel;
import com.yunzhi.shigongli.entity.TaHotelAccount;
import com.yunzhi.shigongli.entity.TaPerson;
import com.yunzhi.shigongli.mapper.TaHotelAccountLogMapper;
import com.yunzhi.shigongli.mapper.TaHotelAccountMapper;
import com.yunzhi.shigongli.mapper.TaHotelMapper;
import com.yunzhi.shigongli.service.ITaHotelService;

import com.yunzhi.shigongli.vo.HotelCommissionVO;
import com.yunzhi.shigongli.vo.HotelListVO;
import com.yunzhi.shigongli.vo.HotelQueryVO;
import com.yunzhi.shigongli.vo.ShopCommissionVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 民宿 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Service
public class TaHotelServiceImpl extends BaseServiceImpl<TaHotelMapper, TaHotel> implements ITaHotelService {

    @Autowired
    TaHotelMapper hotelMapper;

    @Autowired
    TaHotelAccountMapper accountMapper;

    @Autowired
    TaHotelAccountLogMapper hotelAccountLogMapper;

    @Override
    public List<TaHotel> getByName(String hotelName) {
        LambdaQueryWrapper<TaHotel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaHotel::getHotelName, hotelName)
                .gt(TaHotel::getStatus, Constants.STATUS_DELETED)
                .orderByDesc(TaHotel::getCreateDate);
        return list(queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean createNew(TaHotel taHotel) throws Exception {
        save(taHotel);

        TaHotelAccount account = new TaHotelAccount();
        account.setHotelId(taHotel.getHotelId());
        accountMapper.insert(account);

        return true;
    }

    @Override
    public IPage<HotelListVO> getVOListBy(IPage<HotelListVO> pg, String hotelName) {
        if (null != pg) {
            return hotelMapper.getVOPageBy(pg, hotelName);
        } else {
            List<HotelListVO> list = baseMapper.getVOListBy(hotelName);
            IPage<HotelListVO> result = new Page<>();
            result.setRecords(list);
            return result;
        }

    }

    @Override
    public List<TaHotel> getListByPerson(TaPerson taPerson) {
        return hotelMapper.getListByPerson(taPerson.getPersonId());
    }

    @Override
    public IPage<HotelQueryVO> queryHotel(IPage<HotelQueryVO> pg, String hotelName, String startDate, String endDate) {
        if (null != pg) {
            return baseMapper.queryHotelPaged(pg, hotelName, startDate, endDate);
        } else {
            List<HotelQueryVO> list = baseMapper.queryHotelList(hotelName, startDate, endDate);
            IPage<HotelQueryVO> result = new Page<>();
            result.setRecords(list);
            return result;
        }
    }

    @Override
    public IPage<HotelCommissionVO> getCommissionList(IPage<HotelCommissionVO> pg, String hotelName, LocalDateTime startDate, LocalDateTime endDate) {
        if (null != pg) {
            return hotelAccountLogMapper.getHotelAccountPage(pg, Constants.ACCOUNT_COMMISSION, hotelName, startDate, endDate);
        } else {
            IPage<HotelCommissionVO> result = new Page<>();
            List<HotelCommissionVO> list = hotelAccountLogMapper.getHotelAccountList(Constants.ACCOUNT_COMMISSION, hotelName, startDate, endDate);
            result.setRecords(list);
            return result;
        }
    }
}
