package com.yunzhi.shigongli.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.WxPayUtils;
import com.yunzhi.shigongli.entity.*;
import com.yunzhi.shigongli.mapper.*;
import com.yunzhi.shigongli.service.ITaOrderSubService;

import com.yunzhi.shigongli.vo.MineOrder;
import com.yunzhi.shigongli.vo.MineOrderSummary;
import com.yunzhi.shigongli.vo.OrderSubListVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 子订单表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Service
public class TaOrderSubServiceImpl extends BaseServiceImpl<TaOrderSubMapper, TaOrderSub> implements ITaOrderSubService {

    @Autowired
    WxPayUtils wxPayUtils;

    @Autowired
    TaOrderMapper taOrderMapper;

    @Autowired
    TaOrderSubMapper taOrderSubMapper;

    @Autowired
    TaOrderCommissionMapper commissionMapper;

    @Autowired
    TaPersonRecommenderMapper recommenderMapper;

    @Autowired
    TaShopAccountMapper shopAccountMapper;

    @Autowired
    TaShopAccountLogMapper shopAccountLogMapper;

    @Autowired
    TaHotelAccountMapper hotelAccountMapper;

    @Autowired
    TaHotelAccountLogMapper hotelAccountLogMapper;

    /**
     * 订单项目退款
     * @param person
     * @param taOrder
     * @param taOrderSub
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void refund(TaPerson person, TaOrder taOrder, TaOrderSub taOrderSub) throws Exception {
        // 更新表
        taOrderSubMapper.updateStatus(taOrderSub.getSubOrderId(), Constants.ORDER_REFUND);

        // 后面的业务, 比如退款之后, 扣除个人账户之类的
        // 交给退款成功之后的回调处理
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void validation(TaPerson taPerson, TaOrder taOrder, TaOrderSub taOrderSub) throws Exception {
        // 先更新状态
        taOrderSubMapper.updateVerified(taOrderSub.getSubOrderId(), 1);

        // 更新店铺账户
        updateShopAccount(taOrderSub);

        // 获取当前人员的推荐人
        TaPersonRecommender recommender = recommenderMapper.getAcitveRecommender(taPerson.getPersonId());
        if (null != recommender) {
            // 查询分佣表
            List<TaOrderCommission> commissionList = commissionMapper.getListByOrderSub(taOrderSub.getSubOrderId());

            for (TaOrderCommission commission : commissionList) {
                // 流量抽成
                if (Constants.COMMISSION_TRAFFIC.equals(commission.getCommissionType())) {
                    if (Constants.RECOMMEND_SHOP.equals(recommender.getFromType())) {
                        // 如果是商户推荐的
                        createShopAccountLog(recommender.getRecommender(), commission);
                    } else if (Constants.RECOMMEND_HOTEL.equals(recommender.getFromType())) {
                        // 如果是民宿推荐的
                        createHotelAccountLog(recommender.getRecommender(), commission);
                    }
                }

                // 平台抽成
                if (Constants.COMMISSION_PLATFORM.equals(commission.getCommissionType())) {
                    // 平台账户暂时用民宿代替
                    createHotelAccountLog(Constants.SYSTEM_ACCOUNT_ID, commission);
                }
            }
        }
    }

    private void updateShopAccount(TaOrderSub taOrderSub) throws Exception {
        TaShopAccount account = shopAccountMapper.selectByShop(taOrderSub.getShopId());

        // 提点金额
        int row = shopAccountMapper.updateCharges(account.getAccountId(), "commision_charges", taOrderSub.getCommissionCharge());
        if (row < 1) {
            throw new Exception("更新账户提点信息失败");
        }

        // 入明细
        TaShopAccountLog accountLog = new TaShopAccountLog();
        accountLog.setAccountId(account.getAccountId());
        accountLog.setShopId(taOrderSub.getShopId());
        accountLog.setSignType(Constants.ACCOUNT_EXPENSE);
        accountLog.setAmountType(Constants.ACCOUNT_DIVIDE);
        accountLog.setAmount(taOrderSub.getCommissionCharge());
        accountLog.setTargetType(Constants.TARGET_ORDER_SUB);
        accountLog.setTargetId(taOrderSub.getSubOrderId());
        accountLog.setOrderId(taOrderSub.getOrderId());
        accountLog.setSubOrderId(taOrderSub.getSubOrderId());
        accountLog.setCreateDate(LocalDateTime.now());
        shopAccountLogMapper.insert(accountLog);

        // 结算金额 = 订单金额 - 返现 - 提成 = 商户实际的入账
        Integer salesCharge = taOrderSub.getCharges() - taOrderSub.getCashback() - taOrderSub.getCommissionCharge();
        row = shopAccountMapper.updateCharges(account.getAccountId(), "income_charges", salesCharge);
        if (row < 1) {
            throw new Exception("更新账户结算信息失败");
        }

        // 入明细
        accountLog = new TaShopAccountLog();
        accountLog.setAccountId(account.getAccountId());
        accountLog.setShopId(taOrderSub.getShopId());
        accountLog.setSignType(Constants.ACCOUNT_INCOME);
        accountLog.setAmountType(Constants.ACCOUNT_CHARGE);
        accountLog.setAmount(salesCharge);
        accountLog.setTargetType(Constants.TARGET_ORDER_SUB);
        accountLog.setTargetId(taOrderSub.getSubOrderId());
        accountLog.setOrderId(taOrderSub.getOrderId());
        accountLog.setSubOrderId(taOrderSub.getSubOrderId());
        accountLog.setCreateDate(LocalDateTime.now());
        shopAccountLogMapper.insert(accountLog);
    }

    @Override
    public IPage<MineOrder> getOrderSubListOf(IPage<MineOrder> pg, TaPerson taPerson, Integer status, String orderId, Integer isVerified) {
        return baseMapper.getOrderSubListOf(pg, taPerson.getPersonId(), Constants.TARGET_SHOP, status, orderId, isVerified);
    }

    @Override
    public MineOrderSummary getOrderSummary(TaPerson taPerson) {
        MineOrderSummary orderSummary = new MineOrderSummary();

        int unPayNum = taOrderMapper.getOrderNum(taPerson.getPersonId(), Constants.ORDER_READY);
        orderSummary.setUnPayNum(unPayNum);

        int unVerifiedNum = taOrderSubMapper.getOrderNum(taPerson.getPersonId(), Constants.ORDER_PAYED, 0);
        orderSummary.setUnVerifiedNum(unVerifiedNum);

        int refundNum = taOrderSubMapper.getOrderNum(taPerson.getPersonId(), Constants.ORDER_REFUND, null);
        orderSummary.setRefundNum(refundNum);

        return orderSummary;
    }

    @Override
    public List<TaOrderSub> getListByOrder(TaOrder taOrder) {
        return taOrderSubMapper.getListByOrder(taOrder.getOrderId());
    }

    @Override
    public Integer getTotalCharges(String targetType, String targetId, LocalDateTime start, LocalDateTime end) {
        return taOrderSubMapper.getTotalCharges(targetType, targetId, start, end);
    }

    @Override
    public MineOrder getSingleOrderSubOf(TaPerson taPerson, Integer subOrderId) {
        return baseMapper.getSingleOrderSubOf(taPerson.getPersonId(), Constants.TARGET_SHOP, subOrderId);
    }

    @Override
    public IPage<OrderSubListVO> getAdminPagedBy(IPage<OrderSubListVO> pg, String shopName, String description, LocalDateTime startDate, LocalDateTime endDate, String orderId, Integer status, String phone) {
        if (null != pg) {
            return baseMapper.getAdminPagedBy(pg, shopName, description, startDate, endDate, orderId, status, phone);
        } else {
            IPage<OrderSubListVO> result = new Page<>();
            List<OrderSubListVO> list = baseMapper.getAdminListBy(shopName, description, startDate, endDate, orderId, status, phone);
            result.setRecords(list);
            return result;
        }
    }

    /**
     * 商铺抽成入账户
     * @param shopId
     * @param commission
     * @throws Exception
     */
    private void createShopAccountLog(String shopId, TaOrderCommission commission) throws Exception {
        TaShopAccount account = shopAccountMapper.selectByShop(shopId);

        // 先插入明细
        TaShopAccountLog accountLog = new TaShopAccountLog();
        accountLog.setAccountId(account.getAccountId());
        accountLog.setShopId(shopId);
        accountLog.setSignType(commission.getCharge() > 0 ? Constants.ACCOUNT_INCOME : Constants.ACCOUNT_EXPENSE);
        accountLog.setAmountType(Constants.ACCOUNT_COMMISSION);
        accountLog.setAmount(commission.getCharge());
        accountLog.setTargetType(Constants.TARGET_ORDER);
        accountLog.setTargetId(commission.getOrderId());
        accountLog.setOrderId(commission.getOrderId());
        accountLog.setSubOrderId(commission.getSubOrderId());
        accountLog.setCreateDate(LocalDateTime.now());
        shopAccountLogMapper.insert(accountLog);

        // 再累计到账户
        shopAccountMapper.updateBrokerage(account.getAccountId(), accountLog.getAmount());
    }

    /**
     * 民宿抽成入账户
     * @param hotelId
     * @param commission
     * @throws Exception
     */
    private void createHotelAccountLog(String hotelId, TaOrderCommission commission) throws Exception {
        TaHotelAccount account = hotelAccountMapper.selectByHotel(hotelId);

        // 先插入明细
        TaHotelAccountLog accountLog = new TaHotelAccountLog();
        accountLog.setAccountId(account.getAccountId());
        accountLog.setHotelId(hotelId);
        accountLog.setSignType(commission.getCharge() > 0 ? Constants.ACCOUNT_INCOME : Constants.ACCOUNT_EXPENSE);
        accountLog.setAmountType(Constants.ACCOUNT_COMMISSION);
        accountLog.setAmount(commission.getCharge());
        accountLog.setTargetType(Constants.TARGET_ORDER);
        accountLog.setTargetId(commission.getOrderId());
        accountLog.setOrderId(commission.getOrderId());
        accountLog.setSubOrderId(commission.getSubOrderId());
        accountLog.setCreateDate(LocalDateTime.now());
        hotelAccountLogMapper.insert(accountLog);

        // 再累计到账户
        hotelAccountMapper.updateCommission(account.getAccountId(), accountLog.getAmount());
    }
}
