package com.yunzhi.shigongli.service.impl;

import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.entity.TaPerson;
import com.yunzhi.shigongli.entity.TaShare;
import com.yunzhi.shigongli.entity.TaShareOpen;
import com.yunzhi.shigongli.entity.TaTravelPerson;
import com.yunzhi.shigongli.event.OpenShareEvent;
import com.yunzhi.shigongli.mapper.TaShareMapper;
import com.yunzhi.shigongli.mapper.TaShareOpenMapper;
import com.yunzhi.shigongli.mapper.TaTravelPersonMapper;
import com.yunzhi.shigongli.service.ITaShareOpenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * <p>
 * 分享表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Service
public class TaShareOpenServiceImpl extends BaseServiceImpl<TaShareOpenMapper, TaShareOpen> implements ITaShareOpenService {

    @Autowired
    ApplicationEventPublisher eventPublisher;

    @Autowired
    TaShareMapper taShareMapper;

    @Autowired
    TaTravelPersonMapper taTravelPersonMapper;

    @Override
    public void saveInfo(TaPerson taPerson, String shareId) {
//        TaShare taShare = taShareMapper.selectById(shareId);
//        if (null == taShare) return;

        TaShareOpen taShareOpen = new TaShareOpen();
        taShareOpen.setShareId(shareId);
        taShareOpen.setPersonId(taPerson.getPersonId());
        taShareOpen.setCreateDate(LocalDateTime.now());

        save(taShareOpen);

        //
        eventPublisher.publishEvent(new OpenShareEvent(this, taShareOpen));
    }

    @Async
    @Override
    public void checkAndUpdateTravel(TaShareOpen taShareOpen) {
        TaShare taShare = taShareMapper.selectById(taShareOpen.getShareId());
        if (null == taShare) return;

        if (!Constants.TARGET_TRAVEL.equals(taShare.getTargetType())) return ;

        // 分享的对象
        String targetId = taShare.getTargetId();
        Integer serialNo = Integer.parseInt(targetId);
        TaTravelPerson taTravelPerson = taTravelPersonMapper.selectById(serialNo);
        if (null == taTravelPerson || !taTravelPerson.getIsHidden()) return;

        taTravelPerson.setIsHidden(false);
        taTravelPersonMapper.updateById(taTravelPerson);
    }
}
